# Sessions

*NaifJs* is a multi-user dialog manager. 

It can be used as a "server" that manage dialogues by many users, in parallel. Even if used as an embedded library, the engine can manage different conversational sessions at once.

## Session ids

A request, the input sentence coming from a user, is identified by a *user id* (a pure number or any unique identifier string defined by the caller application).

> Note that *user id*, *session id*, or simply *id* are synonyms.

```javascript
  let user = "134824744"
  let sentence = "I love you!" 

  naif.request(user, sentence)
```

## Session storage

To manage multi users at the same time, for each *user id* the engine store some dialog data (at first, the *stateid* conversation state) into a *session* data structure. Each session contains also dialog units variables persistence. 

The sessions database is currently managed as in-memory object, made persistent as JSON file. 
When the engine start (`up`), checks if does exist the session JSON file and load it in memory.
When the engine stop working (`down`) the in-memory sessions are saved again on the disk.
By default, if not specified by developer, the engine look at / create a `sessions.json` in the application directory. 

> If *NaifJs* is used for a single-user application (e.g. an embedded system/IoT device), nothing change. 
> In this specific case the session storage will manage just a single entry.

Here below a session file example, containing entries for two users (session ids are in this case: `134824744` and `103034393`). 

```
$ cat examples/story_it/sessions.js

  "134824744": {
    "unit": "secondUnit",
    "state": "restart",
    "time": 1599501993,
    "active": false,

    "firstUnit": {
      "firstName": "Giorgio"
    }
  }
},
  "103034393": {
    "unit": "ascoltaeripeti",
    "state": "restart",
    "time": 1561539134,
    "active": false,

    "leggieripeti": {
      "word": "TASCHE",
      "retryNum": 5,
      "score": 0
    },

    "ascoltaescrivi": {
      "word": "MERCATI",
      "retryNum": 5,
      "score": 0
    }
  }
}  
```

---

[top](#) | [home](../README.md)
