const { startTimeout } = require('./timeouts')
const { validateTimeout } = require('./messaging/validateTimeout')
const { validateResponseData } = require('./messaging/validateResponseData')
const { getExternalResponse } = require('./externalResponse')
const { getSessionId } = require('./sessionid')
const log = require('./log')

/**
 * response
 *
 * send a message to user, through an external function
 * set timeout callback if arg is specificed
 *
 * @param {Object} TimeoutObject
 * @param {function} TimeoutObject.callback  function pointer
 * @param {ParamsObject} TimeoutObject.params parameters object 
 * @param {number} TimeoutObject.milliseconds number of timeout milliseconds 
 *
 * @param {String|DataObject} data - response data. Mandatory param
 * @param {TimeoutObject} timeout - optional param 
 * @param {String} sessionid - optional param
 *
 *
 * @see https://www.jstips.co/en/javascript/passing-arguments-to-callback-functions/
 * @return {function} externalResponse
 *
 */
function response(data, timeout, sessionid=getSessionId()) {

  const validatedData = validateResponseData(data)

  if (logFilename)
    log.out(sessionid, `[${stateid()}] ${validatedData.text}`)

  // start a timeout if argument 'timeout' is specificed
  if (timeout) {
    const { callback, params, milliseconds } = validateTimeout(timeout)

    startTimeout(callback, milliseconds, params, sessionid)
  }  

  // run the callback, after a data object validation
  return getExternalResponse()(sessionid, validatedData)
}

module.exports = { 

  response,
  say: response, // alias
  ask: response, // alias
  prompt: response, // alias
  reply: response, // alias

}

